@@ -8,9 +8,10 @@ package Tickit::Pen;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 use Carp;
+use Scalar::Util qw( weaken );
 
 our @ALL_ATTRS = qw( fg bg b u i rv af );
 
@@ -31,7 +32,8 @@ Supports the following named pen attributes:
 =item bg => COL
 
 Foreground or background colour. C<COL> may be an integer C<0-7> or one of the
-eight colour names.
+eight colour names. May also be prefixed by C<hi-> for the high-intensity
+version (may not be supported by all terminals).
 
 =item b => BOOL
 
@@ -85,7 +87,9 @@ sub new_from_attrs
    my $class = shift;
    my ( $attrs ) = @_;
 
-   my $self = bless {}, $class;
+   my $self = bless {
+      _on_changed => [],
+   }, $class;
 
    $self->chattrs( $attrs );
 
@@ -120,7 +124,7 @@ sub getattrs
 {
    my $self = shift;
 
-   return %$self;
+   return map { $_ => $self->{$_} } grep { !m/^_/ } keys %$self;
 }
 
 =head2 $pen->chattr( $attr, $value )
@@ -138,7 +142,13 @@ sub chattr
    my $canonicalise = $self->can( "_canonicalise_$attr" );
    $val = $self->$canonicalise( $val ) if $canonicalise;
 
+   # Optimise
+   my $curval = $self->{$attr};
+   return if !defined $curval and !defined $val;
+   return if  defined $curval and  defined $val and $val == $curval;
+
    $self->{$attr} = $val;
+   $self->_changed;
 }
 
 my @COLOURNAMES = qw(
@@ -159,10 +169,12 @@ sub _canonicalise_colour
 
    return undef if !defined $colour;
 
-   return $colour if $colour =~ m/^\d+$/ and $colour < 8;
+   my $high = ( $colour =~ s/^hi-// ) * 8;
+
+   return $high+$colour if $colour =~ m/^\d+$/ and $colour < 8;
 
    foreach my $num ( 0 .. $#COLOURNAMES ) {
-      return $num if $colour eq $COLOURNAMES[$num];
+      return $high+$num if $colour eq $COLOURNAMES[$num];
    }
 
    croak "Unrecognised colour value $colour";
@@ -210,6 +222,54 @@ sub delattr
    my ( $attr ) = @_;
 
    delete $self->{$attr};
+   $self->_changed;
+}
+
+sub _changed
+{
+   my $self = shift;
+
+   $_->on_pen_changed( $self ) for @{ $self->{_on_changed} };
+}
+
+=head2 $pen->add_on_changed( $observer )
+
+Add an observer to the list of objects which will be informed when the pen
+attributes change. The observer will be informed by invoking a method
+
+ $observer->on_pen_changed( $pen )
+
+The observer object is stored weakly, so it is safe to add the
+C<Tickit::Widget> object that is using the pen as an observer.
+
+=cut
+
+sub add_on_changed
+{
+   my $self = shift;
+   my ( $observer ) = @_;
+
+   push @{ $self->{_on_changed} }, $observer;
+   weaken $self->{_on_changed}[-1];
+}
+
+=head2 $pen->remove_on_changed( $observer )
+
+Remove an observer previously added by C<add_on_changed>.
+
+=cut
+
+sub remove_on_changed
+{
+   my $self = shift;
+   my ( $observer ) = @_;
+
+   # Can't grep() because that would strengthen weakrefs
+   my $on_changed = $self->{_on_changed};
+   for( my $i = 0; $i < @$on_changed; ) {
+      $i++, next unless $on_changed->[$i] == $observer;
+      splice @$on_changed, $i, 1, ();
+   }
 }
 
 =head1 AUTHOR
