@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
 
 package Tickit::SingleChildWidget;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Tickit::ContainerWidget );
 
-our $VERSION = '0.29_002';
+our $VERSION = '0.04';
 
 use Carp;
 
@@ -49,7 +49,7 @@ sub new
 
    my $self = $class->SUPER::new( %args );
 
-   $self->set_child( $args{child} ) if exists $args{child};
+   $self->add( $args{child} ) if exists $args{child};
 
    return $self;
 }
@@ -67,46 +67,14 @@ Returns the contained child widget.
 sub child
 {
    my $self = shift;
-   return $self->{child};
+   return ( $self->children )[0];
 }
 
-sub children
-{
-   my $self = shift;
-   my $child = $self->child;
-   return $child ? ( $child ) : () if wantarray;
-   return $child ? 1 : 0;
-}
-
-=head2 $widget->set_child( $child )
-
-Sets the child widget, or C<undef> to remove.
-
-=cut
-
-sub set_child
-{
-   my $self = shift;
-   my ( $child ) = @_;
-
-   if( my $old_child = $self->child ) {
-      undef $self->{child};
-      $self->SUPER::remove( $old_child );
-   }
-
-   $self->{child} = $child;
-
-   if( $child ) {
-      $self->SUPER::add( $child );
-   }
-}
-
-# TODO: This probably isn't needed any more and should be deleted soon
 sub add
 {
    my $self = shift;
    croak "Already have a child; cannot add another" if $self->child;
-   $self->set_child( $_[0] );
+   $self->SUPER::add( @_ );
 }
 
 =head1 AUTHOR
