@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
 
 package Tickit::Widget::Frame;
 
@@ -9,15 +9,12 @@ use strict;
 use warnings;
 use base qw( Tickit::SingleChildWidget );
 
-use Tickit::WidgetRole::Alignable name => "title_align";
-use Tickit::WidgetRole::Penable name => "frame";
-
-our $VERSION = '0.17_003';
+our $VERSION = '0.04';
 
 use Carp;
 
 use Tickit::Pen;
-use Tickit::Utils qw( textwidth substrwidth );
+use Tickit::Utils qw( textwidth substrwidth align );
 
 =head1 NAME
 
@@ -28,8 +25,12 @@ C<Tickit::Widget::Frame> - draw a frame around another widget
  use Tickit;
  use Tickit::Widget::Frame;
  use Tickit::Widget::Static;
+ use IO::Async::Loop;
+ 
+ my $loop = IO::Async::Loop->new;
  
  my $tickit = Tickit->new;
+ $loop->add( $tickit );
  
  my $hello = Tickit::Widget::Static->new(
     text   => "Hello, world",
@@ -60,7 +61,7 @@ This container widget draws a frame around a single child widget.
 Constructs a new C<Tickit::Widget::Static> object.
 
 Takes the following named arguments in addition to those taken by the base
-L<Tickit::SingleChildWidget> constructor:
+L<Tickit::Widget::SingleChildWidget> constructor:
 
 =over 8
 
@@ -90,7 +91,7 @@ sub new
    my $self = $class->SUPER::new( %args );
 
    $self->set_style( $args{style} || "ascii" );
-   $self->_init_frame_pen;
+   $self->set_frame_pen( Tickit::Pen->new );
    $self->set_title( $args{title} ) if defined $args{title};
    $self->set_title_align( $args{title_align} || 0 );
 
@@ -178,6 +179,12 @@ results in the widget being redrawn if the widget has a window associated.
 
 =cut
 
+sub frame_pen
+{
+   my $self = shift;
+   return $self->{frame_pen};
+}
+
 =head2 $widget->set_frame_pen( $pen )
 
 Set a new C<Tickit::Pen> object. This is stored by reference; changes to the
@@ -186,17 +193,15 @@ shared by more than one widget; updates will affect them all.
 
 =cut
 
-sub on_pen_changed
+sub set_frame_pen
 {
    my $self = shift;
-   my ( $pen ) = @_;
+   my ( $newpen ) = @_;
+   return if $self->{frame_pen} and $self->{frame_pen} == $newpen;
 
-   if( $self->window and $pen == $self->frame_pen ) {
-      $self->redraw;
-   }
-   else {
-      $self->SUPER::on_pen_changed( @_ );
-   }
+   $self->{frame_pen}->remove_on_changed( $self ) if $self->{frame_pen};
+   $self->{frame_pen} = $newpen;
+   $newpen->add_on_changed( $self );
 }
 
 =head2 $title = $frame->title
@@ -225,15 +230,39 @@ sub set_title
 
 =head2 $title_align = $frame->title_align
 
+=cut
+
+sub title_align
+{
+   my $self = shift;
+   return $self->{title_align};
+}
+
 =head2 $frame->set_title_align( $title_align )
 
 Accessor for the C<title_align> property. Gives a vlaue in the range C<0.0> to
 C<1.0> to align the title in the top of the frame.
 
-See also L<Tickit::WidgetRole::Alignable>.
+The symbolic values C<left>, C<centre> and C<right> can be supplied instead of
+C<0.0>, C<0.5> and C<1.0> respectively.
 
 =cut
 
+sub set_title_align
+{
+   my $self = shift;
+   my ( $align ) = @_;
+
+   # Convert symbolics
+   $align = 0.0 if $align eq "left";
+   $align = 0.5 if $align eq "centre";
+   $align = 1.0 if $align eq "right";
+
+   $self->{title_align} = $align;
+
+   $self->redraw;
+}
+
 sub children_changed { shift->set_child_window }
 sub reshape          { shift->set_child_window }
 
@@ -244,82 +273,54 @@ sub set_child_window
    my $window = $self->window or return;
    my $child  = $self->child  or return;
 
-   my $lines = $window->lines;
-   my $cols  = $window->cols;
-
-   if( $lines > 2 and $cols > 2 ) {
-      if( my $childwin = $child->window ) {
-         $childwin->change_geometry( 1, 1, $lines - 2, $cols - 2 );
-      }
-      else {
-         my $childwin = $window->make_sub( 1, 1, $lines - 2, $cols - 2 );
-         $child->set_window( $childwin );
-      }
+   if( my $childwin = $child->window ) {
+      $childwin->change_geometry( 1, 1, $window->lines - 2, $window->cols - 2 );
    }
    else {
-      if( $child->window ) {
-         $child->set_window( undef );
-      }
+      my $childwin = $window->make_sub( 1, 1, $window->lines - 2, $window->cols - 2 );
+      $child->set_window( $childwin );
    }
 }
 
-use constant CLEAR_BEFORE_RENDER => 0;
-
 sub render
 {
    my $self = shift;
-   my %args = @_;
 
    my $win = $self->window or return;
-   $win->is_visible or return;
-   my $rect = $args{rect};
 
-   my $cols  = $win->cols;
-   my $lines = $win->lines;
+   my $cols = $win->cols;
 
    my $style = $STYLES{$self->{style}};
 
-   my $framepen = $self->frame_pen;
+   my %frameattrs = $self->{frame_pen}->getattrs;
 
-   foreach my $line ( $rect->top .. $rect->bottom - 1 ) {
-      $win->goto( $line, 0 );
+   $win->goto( 0, 0 );
+   if( defined( my $title = $self->title ) ) {
+      # At most we can fit $cols-4 columns of title
+      my ( $left, $titlewidth, $right ) = align( textwidth( $title ), $cols - 4, $self->{title_align} );
 
-      if( $line == 0 ) {
-         # Top line
-         if( defined( my $title = $self->title ) ) {
-            # At most we can fit $cols-4 columns of title
-            my ( $left, $titlewidth, $right ) = $self->_title_align_allocation( textwidth( $title ), $cols - 4 );
-
-            $win->print( $style->[CORNER_TL] . ( $style->[TOP] x $left ) . " ", $framepen );
-            $win->print( $title, $framepen );
-            $win->print( " " . ( $style->[TOP] x $right ) . $style->[CORNER_TR], $framepen ) if $cols > 1;
-         }
-         else {
-            my $str = $style->[CORNER_TL];
-            $str .= $style->[TOP] x ($cols - 2) if $cols > 2;
-            $str .= $style->[CORNER_TR] if $cols > 1;
-
-            $win->print( $str, $framepen );
-         }
-      }
-      elsif( $line < $lines - 1 ) {
-         # Middle line
-         $win->print( $style->[LEFT], $framepen );
-
-         next if $cols == 1;
-
-         $win->goto( $line, $cols - 1 );
-         $win->print( $style->[RIGHT], $framepen );
-      }
-      else {
-         # Bottom line
-         my $str = $style->[CORNER_BL];
-         $str .= $style->[BOTTOM] x ($cols - 2) if $cols > 2;
-         $str .= $style->[CORNER_BR] if $cols > 1;
-
-         $win->print( $str, $framepen );
-      }
+      $win->penprint( $style->[CORNER_TL] . ( $style->[TOP] x $left ) . " ", %frameattrs );
+      $win->penprint( $title, %frameattrs );
+      $win->penprint( " " . ( $style->[TOP] x $right ) . $style->[CORNER_TR], %frameattrs );
    }
+   else {
+      $win->penprint( $style->[CORNER_TL] .
+                   ( $style->[TOP] x ($cols - 2) ) .
+                   $style->[CORNER_TR],
+                   %frameattrs );
+   }
+
+   foreach my $line ( 1 .. $win->lines - 2 ) {
+      $win->goto( $line, 0 );
+      $win->penprint( $style->[LEFT], %frameattrs );
+
+      $win->goto( $line, $cols - 1 );
+      $win->penprint( $style->[RIGHT], %frameattrs );
+   }
+
+   $win->goto( $win->lines - 1, 0 );
+   $win->penprint( $style->[CORNER_BL] . ( $style->[BOTTOM] x ($cols - 2) ) . $style->[CORNER_BR],
+                   %frameattrs );
 }
 
 =head1 TODO
