@@ -1,15 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
 
 package Tickit::WidgetRole::Borderable;
 
 use strict;
 use warnings;
-use base qw( Tickit::WidgetRole );
 
-our $VERSION = '0.29_002';
+our $VERSION = '0.04';
 
 =head1 NAME
 
@@ -25,22 +24,8 @@ border around the widget's contents.
 
 =head1 METHODS
 
-The following methods are provided on the caller package when this module is
-imported by
-
- use Tickit::WidgetRole::Borderable;
-
 =cut
 
-sub export_subs_for
-{
-   return {
-      map { +$_ => \&$_ }
-         qw( _border_init set_v_border set_h_border set_border get_border_geom ),
-         map { $_, "set_$_" } qw( top_border bottom_border left_border right_border )
-   };
-}
-
 =head2 $widget->_border_init( $argsref )
 
 Initialises the border state from constructor arguments in the referenced
@@ -202,10 +187,12 @@ sub get_border_geom
    my $content_top  = $self->top_border;
    my $content_left = $self->left_border;
 
-   ( my $lines = $window->lines - $content_top  - $self->bottom_border ) > 0 or return;
-   ( my $cols  = $window->cols  - $content_left - $self->right_border  ) > 0 or return;
-
-   return ( $content_top, $content_left, $lines, $cols );
+   return (
+      $content_top,
+      $content_left,
+      $window->lines - $content_top  - $self->bottom_border,
+      $window->cols  - $content_left - $self->right_border,
+   );
 }
 
 =head1 AUTHOR
